import AppKit
import SwiftUI

@main
struct AgentCLIApp: App {
    @NSApplicationDelegateAdaptor(AppDelegate.self) private var appDelegate

    var body: some Scene {
        // Settings are owned by SettingsWindowController so launch stays menu-bar-only.
        Settings {
            EmptyView()
        }
    }
}
